#include "characters.h"

FPSPlayer* FPSPlayer::thePlayer = NULL;
unsigned int PlayerCharacter::frameListenerPriority = 1;
float AlienCharacter::MaxHealth = 350;
CharacterManager* CharacterManager::mSingleton = 0;



class MyTrigger : public Trigger
{
public:
	MyTrigger(const NxString& meshIdentifier, Scene* scene, ShapeDescription* shape, const NxOgre::Pose& pose, ActorParams params = ""):
	  Trigger(meshIdentifier, scene, shape, pose, params)
	  {
	  }

	 void onEnter(Actor* a)
	 {
		 PlayerCharacter* trigger = CharacterManager::getSingleton().getTrigger(this);
		 PlayerCharacter* target = CharacterManager::getSingleton().getTarget(a);
		
		 if(target != trigger && target == FPSPlayer::thePlayer)
			target->addHit(trigger->getHit());

	 }
};

class MyCharMoveController : public CharacterController
{
public:
	MyCharMoveController(float speed = 1.5){movespeed = speed;}
	float movespeed;
	void move(NxVec3 &out, NxVec3 &moveVector, NxQuat &direction, NxVec3 &g, float t, Character*)
	{
		NxVec3 dir = direction.rot(moveVector);
		out = ((dir * movespeed) + g) * t;		
	}
	NxControllerAction onShapeHit(const NxControllerShapeHit & hit)
	{
		return CharacterController::onShapeHit(hit);
	}
	NxControllerAction onControllerHit(const NxControllersHit & hit)
	{
		return CharacterController::onControllerHit(hit);
	}
};

PlayerCharacter::PlayerCharacter(String name, Scene* scene, SceneManager* sceneManager, Vector3 dimensions, Vector3 startPosition, bool debug)
{
	mCurrentAnimState = PLAYERANIMSTATE_STAND;
	isAlive = true;
	mName = name;
	mPlayerDimensions = dimensions;
	mDebug = debug;
	mSceneManager = sceneManager;
	mScene = scene;
	CharacterParams cp;
	cp.setToDefault();
	cp.mDimensions = NxVec3(dimensions.x, dimensions.y, dimensions.z);
	cp.mType = CharacterParams::CT_Box;
	mPlayerCharacter = mScene->createCharacter(mName, startPosition, cp); 
	NxActor* a = mPlayerCharacter->getNxController()->getActor();
	CharacterManager::getSingleton().addTarget((Actor*) a->userData, this);
	mPlayerCharacter->createNode();
	mPlayerNode = mPlayerCharacter->getNode();
	
	mCharacterController = 0;
	mCharacterController = new MyCharMoveController(2.0);
	mPlayerCharacter->setMovementVectorController(mCharacterController);

	if(mDebug)
	{		
		mPlayerBoundDebugNode = mPlayerNode->createChildSceneNode(mName + "_BoundDebugNode");
		mPlayerBoundDebugNode->setScale(Vector3(dimensions.x, dimensions.y, dimensions.z));	
	}

	mCharacterMesh_AnimState = 0;
	mForceAnimChange = false;
	mCharacterMeshEntity = 0;
	mPlayerBoundDebugNode = 0;
		
	this->setPriority(frameListenerPriority++);
	Root::getSingleton().addFrameListener(this);
}

AlienCharacter::AlienCharacter(String name, Scene* scene, SceneManager* sceneManager, Vector3 dimensions, Vector3 startPosition, bool debug)
: PlayerCharacter(name, scene, sceneManager, dimensions, startPosition, debug)
{
	mHealth = MaxHealth;
}

FPSPlayer::FPSPlayer(Scene* scene, SceneManager* sceneManager, Vector3 dimensions, Vector3 startPosition, float cameraHeight, Camera* cam, bool debug)
: PlayerCharacter("FPSPlayerCharacter", scene, sceneManager, dimensions, startPosition, debug)
{	
	mCameraHeight = cameraHeight - dimensions.y / 2.0;
	mCamera = cam;
	mPlayerNode->attachObject(cam); 
	//mCamera->rotate(Vector3(0,1,0), Radian(Degree(180)));
	
	thePlayer = this;	

	mFlashLight = mSceneManager->createLight(mName + "_flashLight");
	mFlashLight->setType(Light::LT_SPOTLIGHT);
	mFlashLight->setCastShadows(true);
    mFlashLight->setDiffuseColour(ColourValue::White);
	mFlashLight->setSpecularColour(ColourValue::White);
	
	mFlashLight->setPosition(-0.1,0.1 + mCameraHeight,0);		
	mFlashLight->setPowerScale(25);
	mFlashLight->setDirection(0,0,-1);
	mPlayerNode->attachObject(mFlashLight);
	mFlashLight->setSpotlightRange(Degree(0),Degree(140));
	mFlashLight->setAttenuation(20.0, 0, 0, 3);
}

FPSPlayer::~FPSPlayer(void)
{
}

void PlayerCharacter::addPlayerMesh(String meshname, float meshscale, bool rotateModel)
{
	mCharacterMeshEntity = mSceneManager->createEntity(mName, meshname);
	mCharacterMeshEntity->setCastShadows(true);
	mMeshScale = meshscale;
	mCurrentAnimState = PLAYERANIMSTATE_STAND;
	mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_STAND));
	mCharacterMesh_AnimState->setEnabled(true);
	mCharacterMesh_AnimState->setLoop(true);
	SceneNode* meshNode = mPlayerNode->createChildSceneNode(mName + "_MeshNode");
	meshNode->attachObject(mCharacterMeshEntity);
	meshNode->setPosition(0 ,-mPlayerDimensions.y / 2.0, 0);
	meshNode->setScale(meshscale, meshscale, meshscale);
	if(rotateModel)
	meshNode->rotate(Vector3::UNIT_Y, Degree(180.0));	
}

void FPSPlayer::addPlayerMesh(String meshname, float meshscale, bool rotateModel)
{	
}

void AlienCharacter::doAction(float dt)
{
	
}

void  AlienCharacter::setAnimState(PlayerAnimState as)
{	
}

String AlienCharacter::getAnimStateName(PlayerAnimState as,  int option)
{	
	return "";
}

bool PlayerCharacter::frameStarted(const FrameEvent& evt)
{
	doAction(evt.timeSinceLastFrame);

	float dt = evt.timeSinceLastFrame;
		
	if(mCharacterMesh_AnimState)
	 mCharacterMesh_AnimState->addTime(dt * getAnimSpeed());	
	
	if(mCharacterMeshEntity)
	this->mCharacterMeshEntity->_updateAnimation();	

	return true;
}

void FPSPlayer::processKeyInputs(InputReader* inputDevice)
{	
	
	mMoving = false;
	if (inputDevice->isKeyDown(KC_S))
	{ 
		mPlayerCharacter->addMovement(Character::DR_Forward); 
		mMoving = true;
	} 	
	if (inputDevice->isKeyDown(KC_W))
	{ 
		mPlayerCharacter->addMovement(Character::DR_Backward);
		mMoving = true;
	} 
	if (inputDevice->isKeyDown(KC_D))
	{ 
		mPlayerCharacter->addMovement(Character::DR_StepRight);
		mMoving = true;
	} 
	if (inputDevice->isKeyDown(KC_A))
	{ 
		mPlayerCharacter->addMovement(Character::DR_StepLeft); 
		mMoving = true;
	} 
	if (inputDevice->isKeyDown(KC_PGUP))
	{ 
		mPlayerCharacter->addMovement(Character::DR_Ladder_Up); 
		mMoving = true;
	} 
	if (inputDevice->isKeyDown(KC_PGDOWN))
	{ 
		mPlayerCharacter->addMovement(Character::DR_Ladder_Down); 
		mMoving = true;
	} 
	if (inputDevice->isKeyDown(KC_LSHIFT))
	{			
		
		if(mCharacterController)
				((MyCharMoveController*)mCharacterController)->movespeed = 2.0;
			mRunning = true;			
	}
	else
	{
		if(mCharacterController)
		((MyCharMoveController*)mCharacterController)->movespeed = 1.0;
		mRunning = false;
	}
	
}

void FPSPlayer::processMouseInputs(InputReader* inputDevice)
{
	Quaternion q; 
	q.FromAngleAxis( Radian(Degree(-inputDevice->getMouseRelX() * 0.13)), Vector3::UNIT_Y ); 
	q = mPlayerCharacter->getGlobalOrientation() * q; 
	mPlayerCharacter->setDirection(q); 

	mCamera->pitch(Radian(Degree(-inputDevice->getMouseRelY() * 0.13)));
	mFlashLight->setDirection(mCamera->getDirection());

	static float lasttime = 0;
	float thistime = Root::getSingleton().getTimer()->getMilliseconds();
	float dt = ((float)(thistime - lasttime))/ 1000.0;
	lasttime = thistime;

	static float flashChangeTime = 0;
	flashChangeTime += dt;
	if( inputDevice->getMouseButton( 0 ) && flashChangeTime > 0.25)
	{
		flashChangeTime = 0;
		mFlashLight->setVisible(!mFlashLight->isVisible());
	}
}

bool FPSPlayer::frameStarted(const FrameEvent& evt)
{
	float dt = evt.timeSinceLastFrame;
	
	static float moveTime=0;
		
	float movecameraup = 0;
	if(mDebug)movecameraup = 2.0;

	if(mMoving)
	{
		moveTime += dt;
		mCamera->setPosition(Vector3(0,mCameraHeight + movecameraup,0) + Vector3(0,0.009 * pow(sin(moveTime*5.0), 2),0));
	}
	else
	{
		moveTime = 0;
		mCamera->setPosition(Vector3(0,mCameraHeight + movecameraup,0));
	}
		
	return true;
}

String FPSPlayer::getAnimStateName(PlayerAnimState as, int option)
{	
	return "";
}

void FPSPlayer::setAnimState(PlayerAnimState as)
{	
}
